@@ -176,12 +176,14 @@ class Custodian(object):
         """
         cwd = os.getcwd()
 
-        with ScratchDir(self.scratch_dir):
-
+        with ScratchDir(self.scratch_dir, create_symbolic_link=True,
+                        copy_to_current_on_exit=True,
+                        copy_from_current_on_enter=True):
             total_errors = 0
             unrecoverable = False
             start = datetime.datetime.now()
-            logging.info("Run started at {}.".format(start))
+            logging.info("Run started at {} in {}.".format(start,
+                                                           self.scratch_dir))
 
             if self.checkpoint:
                 restart, run_log = Custodian._load_checkpoint(cwd)
