@@ -21,23 +21,25 @@ import time
 import datetime
 import operator
 
-from custodian.custodian import ErrorHandler, backup
+from custodian.custodian import ErrorHandler
+from custodian.utils import backup
 from pymatgen.io.vaspio.vasp_input import Poscar, VaspInput
 from pymatgen.transformations.standard_transformations import \
     PerturbStructureTransformation, SupercellTransformation
-from pymatgen.serializers.json_coders import MSONable
 
 from pymatgen.io.vaspio.vasp_output import Vasprun, Oszicar
 from custodian.ansible.intepreter import Modder
 from custodian.ansible.actions import FileActions, DictActions
 
 
-class VaspErrorHandler(ErrorHandler, MSONable):
+class VaspErrorHandler(ErrorHandler):
     """
     Master VaspErrorHandler class that handles a number of common errors
     that occur during VASP runs.
     """
 
+    is_monitor = True
+
     error_msgs = {
         "tet": ["Tetrahedron method fails for NKPT<4",
                 "Fatal error detecting k-mesh",
@@ -65,6 +67,15 @@ class VaspErrorHandler(ErrorHandler, MSONable):
     }
 
     def __init__(self, output_filename="vasp.out"):
+        """
+        Initializes the handler with the output file to check.
+
+        Args:
+            output_filename (str): This is the file where the stdout for vasp
+                is being redirected. The error messages that are checked are
+                present in the stdout. Defaults to "vasp.out", which is the
+                default redirect used by :class:`custodian.vasp.jobs.VaspJob`.
+        """
         self.output_filename = output_filename
         self.errors = set()
 
@@ -192,33 +203,31 @@ class VaspErrorHandler(ErrorHandler, MSONable):
             vi[f].write_file(f)
         return {"errors": list(self.errors), "actions": actions}
 
-    @property
-    def is_monitor(self):
-        return True
-
     def __str__(self):
         return "VaspErrorHandler"
 
-    @property
-    def to_dict(self):
-        return {"@module": self.__class__.__module__,
-                "@class": self.__class__.__name__,
-                "output_filename": self.output_filename}
 
-    @classmethod
-    def from_dict(cls, d):
-        return cls(d["output_filename"])
-
-
-class MeshSymmetryErrorHandler(ErrorHandler, MSONable):
+class MeshSymmetryErrorHandler(ErrorHandler):
     """
     Corrects the mesh symmetry error in VASP. This error is sometimes
     non-fatal. So this error handler only checks at the end of the run,
     and if the run has converged, no error is recorded.
     """
+    is_monitor = False
 
     def __init__(self, output_filename="vasp.out",
                  output_vasprun="vasprun.xml"):
+        """
+        Initializes the handler with the output files to check.
+
+        Args:
+            output_filename (str): This is the file where the stdout for vasp
+                is being redirected. The error messages that are checked are
+                present in the stdout. Defaults to "vasp.out", which is the
+                default redirect used by :class:`custodian.vasp.jobs.VaspJob`.
+            output_vasprun (str): Filename for the vasprun.xml file. Change
+                this only if it is different from the default (unlikely).
+        """
         self.output_filename = output_filename
         self.output_vasprun = output_vasprun
 
@@ -257,32 +266,24 @@ class MeshSymmetryErrorHandler(ErrorHandler, MSONable):
             vi[f].write_file(f)
         return {"errors": ["mesh_symmetry"], "actions": actions}
 
-    @property
-    def is_monitor(self):
-        return False
-
     def __str__(self):
         return "MeshSymmetryErrorHandler"
 
-    @property
-    def to_dict(self):
-        return {"@module": self.__class__.__module__,
-                "@class": self.__class__.__name__,
-                "output_filename": self.output_filename,
-                "output_vasprun": self.output_vasprun}
 
-    @classmethod
-    def from_dict(cls, d):
-        return cls(output_filename=d["output_filename"],
-                   output_vasprun=d["output_vasprun"])
-
-
-class UnconvergedErrorHandler(ErrorHandler, MSONable):
+class UnconvergedErrorHandler(ErrorHandler):
     """
     Check if a run is converged. Switches to ALGO = Normal.
     """
+    is_monitor = False
 
     def __init__(self, output_filename="vasprun.xml"):
+        """
+        Initializes the handler with the output file to check.
+
+        Args:
+            output_vasprun (str): Filename for the vasprun.xml file. Change
+                this only if it is different from the default (unlikely).
+        """
         self.output_filename = output_filename
 
     def check(self):
@@ -317,33 +318,31 @@ class UnconvergedErrorHandler(ErrorHandler, MSONable):
         return {"errors": ["Unconverged"], "actions": actions}
 
     def __str__(self):
-        return "Run unconverged."
-
-    @property
-    def is_monitor(self):
-        return False
-
-    @property
-    def to_dict(self):
-        return {"@module": self.__class__.__module__,
-                "@class": self.__class__.__name__,
-                "output_filename": self.output_filename}
-
-    @classmethod
-    def from_dict(cls, d):
-        return cls(d["output_filename"])
+        return self.__name__
 
 
-class PotimErrorHandler(ErrorHandler, MSONable):
+class PotimErrorHandler(ErrorHandler):
     """
     Check if a run has excessively large positive energy changes.
     This is typically caused by too large a POTIM. Runs typically
     end up crashing with some other error (e.g. BRMIX) as the geometry
     gets progressively worse.
     """
+    is_monitor = True
 
     def __init__(self, input_filename="POSCAR",
                  output_filename="OSZICAR", dE_threshold=1):
+        """
+        Initializes the handler with the input and output files to check.
+
+        Args:
+            input_filename (str): This is the POSCAR file that the run
+                started from. Defaults to "POSCAR". Change
+                this only if it is different from the default (unlikely).
+            output_filename (str): This is the OSZICAR file. Change
+                this only if it is different from the default (unlikely).
+            dE_threshold (float): The threshold energy change. Defaults to 1eV.
+        """
         self.input_filename = input_filename
         self.output_filename = output_filename
         self.dE_threshold = dE_threshold
@@ -377,29 +376,27 @@ class PotimErrorHandler(ErrorHandler, MSONable):
     def __str__(self):
         return "Large positive energy change (POTIM)"
 
-    @property
-    def is_monitor(self):
-        return True
-
-    @property
-    def to_dict(self):
-        return {"@module": self.__class__.__module__,
-                "@class": self.__class__.__name__,
-                "input_filename": self.input_filename,
-                "output_filename": self.output_filename,
-                "dE_threshold": self.dE_threshold}
-
-    @classmethod
-    def from_dict(cls, d):
-        return cls(d["input_filename"], d["output_filename"],
-                   d["dE_threshold"])
-
 
 class FrozenJobErrorHandler(ErrorHandler):
+    """
+    Detects an error when the output file has not been updated
+    in timeout seconds. Perturbs structure and restarts.
+    """
+
+    is_monitor = True
+
     def __init__(self, output_filename="vasp.out", timeout=3600):
         """
-        Detects an error when the output file has not been updated
-        in timeout seconds. Perturbs structure and restarts
+        Initializes the handler with the output file to check.
+
+        Args:
+            output_filename (str): This is the file where the stdout for vasp
+                is being redirected. The error messages that are checked are
+                present in the stdout. Defaults to "vasp.out", which is the
+                default redirect used by :class:`custodian.vasp.jobs.VaspJob`.
+            timeout (int): The time in seconds between checks where if there
+                is no activity on the output file, the run is considered
+                frozen. Defaults to 3600 seconds, i.e., 1 hour.
         """
         self.output_filename = output_filename
         self.timeout = timeout
@@ -427,31 +424,29 @@ class FrozenJobErrorHandler(ErrorHandler):
 
         return {"errors": ["Frozen job"], "actions": actions}
 
-    @property
-    def is_monitor(self):
-        return True
 
-    @property
-    def to_dict(self):
-        return {"@module": self.__class__.__module__,
-                "@class": self.__class__.__name__,
-                "output_filename": self.output_filename,
-                "timeout": self.timeout}
-
-    @classmethod
-    def from_dict(cls, d):
-        return cls(d["output_filename"], timeout=d["timeout"])
-
-
-class NonConvergingErrorHandler(ErrorHandler, MSONable):
+class NonConvergingErrorHandler(ErrorHandler):
     """
     Check if a run is hitting the maximum number of electronic steps at the
     last nionic_steps ionic steps (default=10). If so, change ALGO from Fast to
     Normal or kill the job.
     """
+    is_monitor = True
 
     def __init__(self, output_filename="OSZICAR", nionic_steps=10,
                  change_algo=False):
+        """
+        Initializes the handler with the output file to check.
+
+        Args:
+            output_filename (str): This is the OSZICAR file. Change
+                this only if it is different from the default (unlikely).
+            nionic_steps (int): The threshold number of ionic steps that
+                needs to hit the maximum number of electronic steps for the
+                run to be considered non-converging.
+            change_algo (bool): Whether to attempt to correct the job by
+                changing the ALGO from Fast to Normal.
+        """
         self.output_filename = output_filename
         self.nionic_steps = nionic_steps
         self.change_algo = change_algo
@@ -493,27 +488,6 @@ class NonConvergingErrorHandler(ErrorHandler, MSONable):
     def __str__(self):
         return "NonConvergingErrorHandler"
 
-    @property
-    def is_monitor(self):
-        return True
-
-    @property
-    def to_dict(self):
-        return {"@module": self.__class__.__module__,
-                "@class": self.__class__.__name__,
-                "output_filename": self.output_filename,
-                "nionic_steps": self.nionic_steps,
-                "change_algo": self.change_algo}
-
-    @classmethod
-    def from_dict(cls, d):
-        if "nionic_steps" in d:
-            return cls(output_filename=d["output_filename"],
-                       nionic_steps=d.get("nionic_steps", 10),
-                       change_algo=d.get("change_algo", False))
-        else:
-            return cls(output_filename=d["output_filename"])
-
 
 class PBSWalltimeHandler(ErrorHandler):
     """
@@ -522,8 +496,28 @@ class PBSWalltimeHandler(ErrorHandler):
     the environment (usually the case for PBS systems like most
     supercomputing centers).
     """
+    is_monitor = True
 
-    def __init__(self):
+    # The PBS handler should not terminate as we want VASP to terminate
+    # itself naturally with the STOPCAR.
+    is_terminating = False
+
+    def __init__(self, buffer_time=300):
+        """
+        Initializes the handler with a buffer time.
+
+        Args:
+            buffer_time (int): The min amount of buffer time in secs at the
+                end that the STOPCAR will be written. The STOPCAR is written
+                when the time remaining is < the higher of 3 x the average
+                time for each ionic step and the buffer time. Defaults to
+                300 secs, which is the default polling time of Custodian.
+                This is typically sufficient for the current ionic step to
+                complete. But if other operations are being performed after
+                the run has stopped, the buffer time may need to be increased
+                accordingly.
+        """
+        self.buffer_time = buffer_time
         self.start_time = datetime.datetime.now()
 
     def check(self):
@@ -540,10 +534,9 @@ class PBSWalltimeHandler(ErrorHandler):
                 time_per_step = 0
 
             # If the remaining time is less than average time for 3 ionic
-            # steps or 300 seconds (last 5 minutes, which is the
-            # default monitoring frequency).
+            # steps or buffer_time.
             time_left = wall_time - total_secs
-            if time_left < max(time_per_step * 3, 300):
+            if time_left < max(time_per_step * 3, self.buffer_time):
                 return True
         return False
 
@@ -560,22 +553,3 @@ class PBSWalltimeHandler(ErrorHandler):
 
     def __str__(self):
         return "PBSWalltimeHandler"
-
-    @property
-    def is_monitor(self):
-        return True
-
-    @property
-    def is_terminating(self):
-        # The PBS handler should not terminate as we want VASP to terminate
-        # itself naturally with the STOPCAR.
-        return False
-
-    @property
-    def to_dict(self):
-        return {"@module": self.__class__.__module__,
-                "@class": self.__class__.__name__}
-
-    @classmethod
-    def from_dict(cls, d):
-        return cls()
